@@ -223,8 +223,7 @@ class ExecutionServer(BaseServer):
             "pending_requests":server_data["pending_requests"],
             "current_timestamp":server_data["current_timestamp"],
             "job_queue":len(self.job_queue),
-            "current_sql":self.current_sql.replace("\n", "")
-        }
+            "current_sql":self.current_sql.replace("\n", "")}
         if self.uuid_limits["start"] is None and self.uuid_limits["end"] is not None:
             attributes["range"] = "Start - %s" % self.uuid_limits["end"]
         elif self.uuid_limits["start"] is not None and self.uuid_limits["end"] is None:
@@ -325,11 +324,28 @@ class ExecutionServer(BaseServer):
             self.uuid_limits = splits[self.peer_uuids.index(self.uuid)]
         else:
             self.uuid_limits = {"start":None, "end":None}
+        job_queue_length = len(self.job_queue)
+        if self.uuid_limits["start"] is None and self.uuid_limits["end"] is not None:
+            self.job_queue = filter(self.testJobByEnd, self.job_queue)
+        elif self.uuid_limits["start"] is not None and self.uuid_limits["end"] is None:
+            self.job_queue = filter(self.testJobByStart, self.job_queue)
+        elif self.uuid_limits["start"] is not None and self.uuid_limits["end"] is not None:
+            self.job_queue = filter(self.testJobByStartAndEnd, self.job_queue)
+        LOGGER.critical("Abandoned %s jobs that were out of range." % (job_queue_length - len(self.job_queue)))
         LOGGER.debug("Updated UUID limits to: %s" % self.uuid_limits)
-        
+    
     def _coordinateErrback(self, error):
         LOGGER.error("Could not query SimpleDB for peers: %s" % str(error))
 
+    def testJobByEnd(self, job):
+        return job["uuid"] < self.uuid_limits["end"]
+
+    def testJobByStart(self, job):
+        return job["uuid"] > self.uuid_limits["start"]
+
+    def testJobByStartAndEnd(self, job):
+        return (job["uuid"] < self.uuid_limits["end"] and job["uuid"] > self.uuid_limits["start"])
+        
     def verifyPeer(self, uuid, peer):
         LOGGER.debug("Verifying peer %s" % uuid)
         deferreds = []
@@ -462,6 +478,8 @@ class ExecutionServer(BaseServer):
         for uuid in data:
             if uuid in self.active_jobs or uuid in self.queued_jobs:
                 continue
+            if len(self.job_queue) > self.reservation_check_interval * 20:
+                break
             kwargs_raw = {}
             reserved_arguments = {}
             # Load attributes into dicts for use by the system or custom functions.
@@ -543,7 +561,7 @@ class ExecutionServer(BaseServer):
             uuid = job["uuid"]
             del self.queued_jobs[uuid]
             # Call the function.
-            LOGGER.debug("Calling %s with args %s" % (function_name, kwargs))
+            LOGGER.critical("Calling %s with args %s" % (function_name, kwargs))
             # Schedule the next request.
             d = self.callExposedFunction(
                 exposed_function["function"], 
